/*
 * basic_cdc.h
 * ----
 * takeover functions for the console to use USB CDC
 */

#include "basic_cdc.h"


/* handlers */
static void usb_cdc_tx_wait()
{
	uint32_t timeout = 100;

	// if a tx is in progress, then wait a while
	// bruteforce hack: timeout added since sometimes this isn't cleared...
	while ((cdc_state & CDC_STATE_TX_BUSY) && timeout--) {
		__WFI();
	}
}

void usb_cdc_putc(char c)
{
	usbTxBuffer[0] = c;
	usbTxBuffer[1] = 0;
	CDC_DataTransmit(USB_STATUS_OK, 0, 0);
	usb_cdc_tx_wait();
}

int usb_cdc_puts(char *c, int cnt)
{
	int i;

	if (!cnt) cnt = strlen(c);
	if (cnt >= (CDC_USB_TX_BUF_SIZ - 1)) cnt = (CDC_USB_TX_BUF_SIZ - 1);

	strncpy((char *)usbTxBuffer, c, cnt);
	CDC_DataTransmit(USB_STATUS_OK, 0, 0);

	// wait until buffer is sent before we continue, so we can send consecutive buffers
	usb_cdc_tx_wait();

	// bruteforce hack: clear old buffer before reuse
	for (i = 0; i < CDC_USB_TX_BUF_SIZ; i++) {
		usbTxBuffer[i] = 0;
	}

	return cnt;
}

int usb_cdc_gets(char *buf, int cnt)
{
	int i;
	int j;

	if (cnt) {
		if (usbRxBufBytes == 0) {
			return 0;
		}

		// copy requested bytes
		for (i = 0; i < cnt && i < (CDC_USB_RX_BUF_SIZ - 1); i++) {
			buf[i] = usbRxBuffer[i];
		}
		buf[i] = 0;

		// shift working buffer to the left
		j = 0;
		while (i < CDC_USB_RX_BUF_SIZ && i < usbRxBufBytes) {
			usbRxBuffer[j] = usbRxBuffer[i];
			i++;
			j++;
		}
		usbRxBuffer[j] = 0;
		usbRxBufBytes = j;
	}

	return cnt;
}
