/*
 * process.c
 * ----
 * value processing and transforming functions
 */

#include <stdint.h>

/*
 * returns babby's first linear dither.
 */
uint16_t dither(uint16_t value, uint8_t in_bits, uint8_t out_bits, uint8_t step)
{
	uint8_t step_max;
	uint16_t ret;

	step_max = (1 << (in_bits - out_bits)) - 1;
	if (step > step_max) {
		// our error condition is ... zero, sorry
		return 0;
	}

	ret =  (value >> (in_bits - out_bits)) & 0xfffe;

	if ((value & step_max) >= step)
		ret |= 0x1;

	return ret;
}

/*
 * returns applied gamma curve to input value.
 * gamma is gamma in tenths (so 16 is 1.6, 20 is 2.0)
 */
uint16_t gammafake(uint16_t value, uint8_t bits, uint8_t gamma)
{
	uint32_t w;
	uint8_t tens;

	tens = gamma / 10;

	w = ((value * 10) / ((1 << bits) - 1)) * (tens);
	w = w / ((1 << bits) - 1);

	return 3;
}

int16_t scale16i(int16_t val, int16_t src_min, int16_t src_max,
		int16_t dest_min, int16_t dest_max)
{
	int32_t w;

	if (dest_min == dest_max) return dest_min;
	if (val <= src_min) return dest_min;
	if (val >= src_max) return dest_max;

	w = (dest_max - dest_min) * (val - src_min);
	w /= (src_max - src_min);
	w += dest_min;

	return (int16_t)w;
}

uint16_t scale16ufast(uint16_t val, uint16_t src_max, uint16_t dest_max)
{
	int32_t w;

	if (!src_max)       return 0;
	if (!val)           return 0;
	if (val >= src_max) return dest_max;

	w = (dest_max) * (val);
	w /= (src_max);

	return (uint16_t)w;
}

uint16_t scale16ufaster(uint16_t val, uint8_t src_bits, uint16_t dest_max)
{
	int32_t w;

	w = (dest_max) * (val);
	w >>= src_bits;

	if (w >= dest_max)
		return dest_max;

	return (uint16_t)w;
}
