/**
 * shiftreg.c
 */

#include "em_device.h"
#include "em_chip.h"

#include "shiftreg.h"


/* shared shiftreg data to send with sr_update */
uint16_t sr_dat[2];

void sr_init()
{
	/* Pin PB11 is configured to Push-pull */
	GPIO->P[1].MODEH = (GPIO->P[1].MODEH & ~_GPIO_P_MODEH_MODE11_MASK)
			| GPIO_P_MODEH_MODE11_PUSHPULL;

	/* Pin PF3 is configured to Push-pull */
	GPIO->P[5].MODEL = (GPIO->P[5].MODEL & ~_GPIO_P_MODEL_MODE3_MASK)
			| GPIO_P_MODEL_MODE3_PUSHPULL;

	/* Pin PF4 is configured to Push-pull */
	GPIO->P[5].MODEL = (GPIO->P[5].MODEL & ~_GPIO_P_MODEL_MODE4_MASK)
			| GPIO_P_MODEL_MODE4_PUSHPULL;

	/* Pin PF5 is configured to Push-pull */
	GPIO->P[5].MODEL = (GPIO->P[5].MODEL & ~_GPIO_P_MODEL_MODE5_MASK)
			| GPIO_P_MODEL_MODE5_PUSHPULL;
}

/* updating functions */
/*
 * sends out bits so 16-bit data should be in the format
 *   MSB(15) sr2_h sr2_g .. sr1_b sr1_a LSB(0)
 */
void sr_send(uint16_t *dat, uint8_t len)
{
	uint16_t mask;

	GPIO->P[SR_LAT_PORT].DOUTCLR = (1 << SR_LAT_PIN);

	if (len > 16) len = 16;

	// clock out data
	while (len) {
		len--;
		mask = (1 << len);

		if (dat[0] & mask) {
			GPIO->P[SR_DAT0_PORT].DOUTSET = (1 << SR_DAT0_PIN);
		} else {
			GPIO->P[SR_DAT0_PORT].DOUTCLR = (1 << SR_DAT0_PIN);
		}

		if (dat[1] & mask) {
			GPIO->P[SR_DAT2_PORT].DOUTSET = (1 << SR_DAT2_PIN);
		} else {
			GPIO->P[SR_DAT2_PORT].DOUTCLR = (1 << SR_DAT2_PIN);
		}

		GPIO->P[SR_CLK_PORT].DOUTSET = (1 << SR_CLK_PIN);
		GPIO->P[SR_CLK_PORT].DOUTCLR = (1 << SR_CLK_PIN);
	}

	// update registers
	GPIO->P[SR_LAT_PORT].DOUTSET = (1 << SR_LAT_PIN);
	GPIO->P[SR_LAT_PORT].DOUTCLR = (1 << SR_LAT_PIN);
}

void sr_update()
{
	sr_send(sr_dat, SR_BITLEN);
}
