/**
 * hw/vibrator.c
 */

#include "vibrator.h"

/* pin config */
#include "InitDevice.h"

/* efm32 specific */
#include "em_gpio.h"


/* patterns */
static const VibratePattern vpattern[VIBRATE_PATTERN_COUNT] = {
		// all patterns should ideally last 16 cycles (one second)
		// call vibrate_process() 16 times per second for correct timing
		{2, {8, 8} 				},	// bzzzz ------
		{4, {3, 2, 3, 8} 		},	// bzz - bzz ------
		{6, {2, 3, 2, 1, 2, 7}	}, 	// bzz --- bzz bzz ------
		{2, {13, 3} 			} 	// bzzzzzzzz --
};

/* handling */
static uint8_t vibrate_pidx;

static uint8_t vibrate_vidx;
static uint8_t vibrate_vcount;
static uint8_t vibrate_vstate;

static uint16_t vibrate_repeat;


/* functions */
void vibrate_init()
{
	// hardware already initialized

	// flags
	vibrate_pidx = 0;
	vibrate_vcount = 0;

	vibrate_repeat = 0;
}

void vibrate_set_output(uint8_t on)
{
	if (on) {
		GPIO->P[VIBRATE_PORT].DOUTSET |= (1 << VIBRATE_PIN);
	} else {
		GPIO->P[VIBRATE_PORT].DOUTCLR |= (1 << VIBRATE_PIN);
	}
}

void vibrate_set_pattern(uint8_t pattern_idx)
{
	if (pattern_idx < VIBRATE_PATTERN_COUNT) {
		vibrate_pidx = pattern_idx;
	}
}

uint8_t vibrate_get_pattern()
{
	return vibrate_pidx;
}

void vibrate_queue(uint16_t repeats)
{
	vibrate_repeat = repeats;

	vibrate_vidx = 0;
	vibrate_vcount = 0;
}

int8_t vibrate_process()
{
	if (vibrate_repeat) {
		if (vibrate_vidx < vpattern[vibrate_pidx].count) {
			if (vibrate_vcount < vpattern[vibrate_pidx].pattern[vibrate_vidx]) {
				vibrate_vstate = vibrate_vidx & 1 ? VIBRATE_OFF : VIBRATE_ON;
				vibrate_vcount++;
			} else {
				vibrate_vcount = 0;
				vibrate_vidx++;
			}
		} else {
			vibrate_repeat--;
			vibrate_vidx = 0;
			vibrate_vcount = 0;
			vibrate_process();

			if (!vibrate_repeat) {
				vibrate_vstate = VIBRATE_OFF;
				return VIBRATE_OFF;
			}
		}

		vibrate_set_output(vibrate_vstate);
		return vibrate_vstate;
	}

	return -1;
}
