/*
 * prog_interrupt.c
 */

#include "em_device.h"
#include "em_rtc.h"

#include "display/led.h"

#include "hw/userio.h"
#include "hw/lightsense.h"
#include "hw/vibrator.h"

#include "rtc/time.h"
#include "rtc/alarm.h"


/* flags */
uint8_t prog_clock_render;
uint8_t prog_alarm_state;


/* shit timekeeper */
volatile uint32_t halfms;


/* global periodic interrupt */
void SysTick_Handler(void)
{
	uint32_t hms;

	int i;

	halfms++;
	hms = halfms;

	// process LEDs
	led_state_next();

	// process user input
	i = userio_update(prog_alarm_state);

	// process alarm going off
	if (prog_alarm_state) {
		if (!i) {
			// alarm has been disabled
			prog_alarm_state = 0;
			vibrate_set_output(0);
			vibrate_queue(0);
		}
	}

	// temporary LED setting shit; remove me
	led_eyes[0][LED_RED] += 4;
	led_eyes[0][LED_RED] %= 10240; // 10000;

	led_eyes[0][LED_GREEN] = led_eyes[0][LED_RED];
	led_eyes[0][LED_BLUE] = led_eyes[0][LED_RED];

	led_eyes[1][LED_RED] = led_eyes[1][LED_GREEN] = led_eyes[1][LED_BLUE] = led_eyes[0][LED_RED];

	for (i = 0; i < 5; i++) {
		led_bone[i][0] = led_bone[i][1] = led_bone[i][2] = led_eyes[0][LED_RED];
		led_bone[i][0] <<= 1;
		led_bone[i][1] <<= 1;
	}


	for (i = 0; i < 8; i++) {
		led_skull[i] = 16;
	}

	// update light value 4 times per second
	if (!prog_alarm_state) {
		lsens_run(((hms % 512) == 0) ? 1 : 0);
	}

	// update LED clock
	if (prog_clock_render) {
		led_render_clock();
		prog_clock_render = 0;
	}

	// clock animation / brightness update and vibrator
	if (hms % 128 == 0) {
		prog_clock_render = 1;
		i = vibrate_process();
		if (prog_alarm_state && i >= 0) {
			lsens_led_output(i);
		}
	}

	// render LEDs
	switch (hms % 32) {
		case 8: 	led_render_eyes(); break;
		case 16: 	led_render_skull(); break;
		case 24:	led_render_bone(); break;
	}
}

/* rtc interrupt */
void RTC_IRQHandler()
{
	RTC_IntClear(RTC_IFS_COMP0);

	if (rtc_tick() == 0) {
		// process alarms
		if (alarm_check()) {
			prog_alarm_state = 1;
			vibrate_queue(120);
		}
	}

	// allow clock to render
	if (rtc_is_clock_set()) {
		prog_clock_render = 1;
	}
}
