/***************************************************************************//**
 * @file  cdc.h
 * @brief USB Communication Device Class (CDC) driver.
 * @version 4.2.1
 *******************************************************************************
 * @section License
 * <b>(C) Copyright 2014 Silicon Labs, http://www.silabs.com</b>
 *******************************************************************************
 *
 * This file is licensed under the Silabs License Agreement. See the file
 * "Silabs_License_Agreement.txt" for details. Before using this software for
 * any purpose, you must agree to the terms of that agreement.
 *
 ******************************************************************************/
#ifndef __CDC_H
#define __CDC_H

#include "em_usb.h"

/***************************************************************************//**
 * @addtogroup Drivers
 * @{
 ******************************************************************************/

/***************************************************************************//**
 * @addtogroup Cdc
 * @{
 ******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif


#define CDC_BULK_EP_SIZE  	(USB_FS_BULK_EP_MAXSIZE) /* This is the max. ep size.    */
#define CDC_USB_RX_BUF_SIZ  CDC_BULK_EP_SIZE /* Packet size when receiving on USB. */
#define CDC_USB_TX_BUF_SIZ  63   			 /* Packet size when transmitting on USB.  */

extern UBUF(usbRxBuffer, CDC_USB_RX_BUF_SIZ);
extern volatile uint16_t usbRxBufBytes;

extern UBUF(usbTxBuffer, CDC_USB_TX_BUF_SIZ);

#define CDC_STATE_TX_BUSY 		0x01
extern volatile uint8_t cdc_state;


void CDC_Init();
int  CDC_SetupCmd(const USB_Setup_TypeDef *setup);
void CDC_StateChangeEvent( USBD_State_TypeDef oldState,
                           USBD_State_TypeDef newState);

int CDC_DataTransmit(USB_Status_TypeDef status, uint32_t xferred, uint32_t remaining);


#ifdef __cplusplus
}
#endif

/** @} (end group Cdc) */
/** @} (end group Drivers) */

#endif /* __CDC_H */
